/**********************************************************************
** Copyright (C) 2000-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qtopia Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#include "batterydata.h"

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>

BatteryDataManager *BatteryDataManager::batteryDataManager = 0;
BatteryData *BatteryDataManager::ps = 0;

BatteryDataManager::BatteryDataManager()
{
  batteryDataManager = this;
  ps = new BatteryData;
}

const BatteryData &BatteryDataManager::readStatus()
{
  
  if ( !batteryDataManager )
    (void)new BatteryDataManager;
  
  
  bool ok = false;
  
  int ac = 0xff;
  int bs = 0xff;
  int bf = 0xff;
  int pc = -1;
  int sec = -1;
  
  FILE *f = fopen("/proc/apm", "r");
  if ( f ) {
    //I 1.13 1.2 0x02 0x00 0xff 0xff 49% 147 sec
    char u;
    fscanf(f, "%*[^ ] %*d.%*d 0x%*x 0x%x 0x%x 0x%x %d%% %i %c",
	   &ac, &bs, &bf, &pc, &sec, &u);
    fclose(f);
    switch ( u ) {
    case 'm': sec *= 60;
    case 's': break; // ok
    default: sec = -1; // unknown
    }
    
    // extract data
    switch ( bs ) {
    case 0x00:
      ps->bs = BatteryData::High;
      break;
    case 0x01:
      ps->bs = BatteryData::Low;
      break;
    case 0x7f:
      ps->bs = BatteryData::VeryLow;
      break;
    case 0x02:
      ps->bs = BatteryData::Critical;
      break;
    case 0x03:
      ps->bs = BatteryData::Charging;
      break;
    case 0x04:
    case 0xff: // 0xff is Unknown but we map to NotPresent
    default:
      ps->bs = BatteryData::NotPresent;
      break;
    }
    
    switch ( ac ) {
    case 0x00:
      ps->ac = BatteryData::Offline;
      break;
    case 0x01:
      ps->ac = BatteryData::Online;
      break;
    case 0x02:
      ps->ac = BatteryData::Backup;
      break;
    }
    
    if ( pc > 100 )
      pc = -1;
    
    ps->percentRemain = pc;
    ps->secsRemain = sec;
    
    ok = true;
  }
  
  return *ps;
}

